/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		Ball.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	26.08.2001
// letzte nderung:	26.08.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	kapselt die Physik und die Engine fr die Blle		
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "Ball.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::Ball
/////////////////////////////////////////////////////////////////////////////////////
// Info:	instanziert ein Ballobjekt und holt Speicher fr die ensprechende
//			Anzahl von Bllen
/////////////////////////////////////////////////////////////////////////////////////
Ball::Ball(int iNrOfBalls)
{
	int iCount;

	m_iNrOfBalls=	iNrOfBalls;
	m_iActualBall=	0;
	m_fGravity=		0;
	m_fFloor=		0;
	m_iDistance=	100000;

	m_pfPosX=		new float[iNrOfBalls];
	m_pfPosY=		new float[iNrOfBalls];
	m_pfPosZ=		new float[iNrOfBalls];

	m_pfDeltaX=		new float[iNrOfBalls];
	m_pfDeltaY=		new float[iNrOfBalls];
	m_pfDeltaZ=		new float[iNrOfBalls];

	m_piTransX=		new int[iNrOfBalls];
	m_piTransY=		new int[iNrOfBalls];
	m_piTransZ=		new int[iNrOfBalls];

	m_piScreenX=	new int[iNrOfBalls];
	m_piScreenY=	new int[iNrOfBalls];

	m_piDeepSort=	new int[iNrOfBalls];

	for(iCount=0;iCount<iNrOfBalls;iCount++)
	{
		m_pfPosX[iCount]=		0;
		m_pfPosY[iCount]=		0;
		m_pfPosZ[iCount]=		0;

		m_pfDeltaX[iCount]=		0;
		m_pfDeltaY[iCount]=		0;
		m_pfDeltaZ[iCount]=		0;

		m_piTransX[iCount]=		0;
		m_piTransY[iCount]=		0;
		m_piTransZ[iCount]=		0;

		m_piScreenX[iCount]=	0;
		m_piScreenY[iCount]=	0;

		m_piDeepSort[iCount]=	iCount;
	}

}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::Ball
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt den Speicher des Objekts wieder frei (!)
/////////////////////////////////////////////////////////////////////////////////////
Ball::~Ball()
{
	delete[] m_pfPosX;
	delete[] m_pfPosY;
	delete[] m_pfPosZ;

	delete[] m_pfDeltaX;
	delete[] m_pfDeltaY;
	delete[] m_pfDeltaZ;

	delete[] m_piTransX;
	delete[] m_piTransY;
	delete[] m_piTransZ;

	delete[] m_piScreenX;
	delete[] m_piScreenY;

	delete[] m_piDeepSort;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::setPos
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Position des nten Balls setzen	
/////////////////////////////////////////////////////////////////////////////////////
void Ball::setPos(int iNr, float fX, float fY, float fZ)
{
	if (iNr>=m_iNrOfBalls || iNr<0)
		return;

	m_pfPosX[iNr]=fX;
	m_pfPosY[iNr]=fY;
	m_pfPosZ[iNr]=fZ;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::setDelta
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt die Bewegungsgeschwindigkeiten des nten Balls
/////////////////////////////////////////////////////////////////////////////////////
void Ball::setDelta(int iNr, float fX, float fY, float fZ)
{
	if (iNr>=m_iNrOfBalls || iNr<0)
		return;

	m_pfDeltaX[iNr]=fX;
	m_pfDeltaY[iNr]=fY;
	m_pfDeltaZ[iNr]=fZ;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::setGravity
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt die Gravitation
/////////////////////////////////////////////////////////////////////////////////////
void Ball::setGravity(float fGrav)
{
	m_fGravity=fGrav;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::setFloor
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt die Hhe des Bodens
/////////////////////////////////////////////////////////////////////////////////////
void Ball::setFloor(float fFloor)
{
	m_fFloor=fFloor;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::setDistance
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt die Entfernung des Koordinatenursprungs
/////////////////////////////////////////////////////////////////////////////////////
void Ball::setDistance(int iDistance)
{
	m_iDistance=iDistance;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::getActualBall
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt die Nummer des ltesten Balls
//			(funktioniert fast wie eine Ringliste)
/////////////////////////////////////////////////////////////////////////////////////
int  Ball::getActualBall()
{
	int iNow=m_iActualBall;

	m_iActualBall++;
	if (m_iActualBall==m_iNrOfBalls)
		m_iActualBall=0;

	return iNow;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::deepSort
/////////////////////////////////////////////////////////////////////////////////////
// Info:	sortiert die Blle der Tiefe nach, tiefster zuerst
//			(benutzt einen modifizierten Bubble-Sort)
/////////////////////////////////////////////////////////////////////////////////////
void Ball::deepSort()
{
	int iCount;
	int iLast;
	int iTmp;

	iLast=m_iNrOfBalls-1;

	do
	{
		iTmp=-999999999;
		
		for (iCount=iLast;iCount>1;iCount--)
		{
			if (m_piTransY[m_piDeepSort[iCount-1]]<m_piTransY[m_piDeepSort[iCount]])
			{
				iTmp=m_piDeepSort[iCount-1];
				m_piDeepSort[iCount-1]=m_piDeepSort[iCount];
				m_piDeepSort[iCount]=iTmp;
			}
		}
		iLast--;
		for (iCount=1;iCount<=iLast;iCount++)
		{
			if (m_piTransY[m_piDeepSort[iCount-1]]<m_piTransY[m_piDeepSort[iCount]])
			{
				iTmp=m_piDeepSort[iCount-1];
				m_piDeepSort[iCount-1]=m_piDeepSort[iCount];
				m_piDeepSort[iCount]=iTmp;
			}
		}
		iLast--;
	} while (iTmp!=-999999999);

}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::moveandtransform
/////////////////////////////////////////////////////////////////////////////////////
// Info:	bewegt, dreht und transformiert die Ballkoordinaten zu 
//			Bildschirmkoordinaten
/////////////////////////////////////////////////////////////////////////////////////
void Ball::moveandtransform(int iAngle, float fTimePassed)
{
	int		iCount;
	float	fAngle=(float)iAngle*0.000314f;

	for (iCount=0;iCount<m_iNrOfBalls;iCount++)
	{

		if (m_pfPosZ[iCount]>m_fFloor)
		{
			m_pfPosZ[iCount]=m_fFloor;
			if (m_pfDeltaZ[iCount]>0.0f)
				m_pfDeltaZ[iCount]=-0.8f*m_pfDeltaZ[iCount];
		}

		m_pfPosX[iCount]+=m_pfDeltaX[iCount]*fTimePassed;
		m_pfPosY[iCount]+=m_pfDeltaY[iCount]*fTimePassed;
		m_pfPosZ[iCount]+=m_pfDeltaZ[iCount]*fTimePassed+m_fGravity*fTimePassed*fTimePassed*0.5f;

		m_pfDeltaZ[iCount]+=m_fGravity*fTimePassed;

		m_piTransX[iCount]=(int)(50.0f* (m_pfPosX[iCount]*cos(fAngle)-m_pfPosY[iCount]*sin(fAngle)));
		m_piTransY[iCount]=(int)(50.0f* (m_pfPosX[iCount]*sin(fAngle)+m_pfPosY[iCount]*cos(fAngle)));
		m_piTransZ[iCount]=(int)(50.0f* m_pfPosZ[iCount]);

		if (m_piTransY[iCount]+m_iDistance>0)
		{
			m_piScreenX[iCount]=496+(500*m_piTransX[iCount])/(m_piTransY[iCount]+m_iDistance);
			m_piScreenY[iCount]=40+(500*m_piTransZ[iCount])/(m_piTransY[iCount]+m_iDistance);
		}
		else
		{
			m_piScreenX[iCount]=-496;
			m_piScreenY[iCount]=-368;
		}
	}

	deepSort();

}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Ball::draw
/////////////////////////////////////////////////////////////////////////////////////
// Info:	zeichnet die Blle auf das Display
/////////////////////////////////////////////////////////////////////////////////////
void Ball::draw(XDisplay* pxdDest, XSprite* pxsSprite, int iNrOfFrames)
{
	int iCount;
	int iFrame;

	for (iCount=0;iCount<m_iNrOfBalls;iCount++)
	{
		pxsSprite->m_lX=m_piScreenX[m_piDeepSort[iCount]];
		pxsSprite->m_lY=m_piScreenY[m_piDeepSort[iCount]];
		iFrame=iNrOfFrames/2+m_piTransY[m_piDeepSort[iCount]]/6000;
		if (iFrame<0) iFrame=0;
		if (iFrame>=iNrOfFrames) iFrame=iNrOfFrames-1;
		pxsSprite->m_lFrame=iFrame;
		pxsSprite->blit(pxdDest);
	}
}



/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	Ball.cpp
/////////////////////////////////////////////////////////////////////////////////////
