/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		main.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	21.08.2001
// letzte nderung:	27.08.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	XGamesSdk Demo Anwendung			
//			demonstriert die grundlegende Anwendung des XGamesSDK, speziell blitten
//			und Timing
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include <math.h>

#include "XGameSDK\XDisplay.h"
#include "XGameSDK\XSurface.h"
#include "XGameSDK\XSprite.h"
#include "XGameSDK\Xfont.h"
#include "XGameSDK\XApplication.h"
#include "XGameSDK\XLogging.h"
#include "XGameSDK\XTimer.h"
#include "XGameSDK\XFramecounter.h"
#include "Ball.h"

//#define XDEBUG

/////////////////////////////////////////////////////////////////////////////////////
//globale Variablen 
XApplication	g_xaApplication;
XDisplay		g_xdDisplay;
XTimer			g_xtTimer;
XFrameCounter	g_xfcFrameCounter;
long			g_lMode;
bool			g_bRunning;
bool			g_bWantToQuit;
bool			g_bESCKey=false;
bool			g_bFCounterOn=false;

const int		g_iNrOfBalls=961;

char*			g_pcScrollText=
"THIS IS A SMALL DEMO BUILD WITH THE XGAMES-SDK.  GET IT AT \
'HTTP://WWW.X-TREME-ONLINE.DE' OR 'HTTP://PC.EXIT.DE/TGGC' !            \
PRESS 'F' TO TOGGLE THE FRAMECOUNTER.       \
NOW HAVE FUN WATCHING SOME KEWL FX.                          \
GREETS FLY TO: XTREME, MEMBERS OF DUS, ROADRUNNER, HOFF, RANDAN, \
UWL /ESPECIALLY KRATZ00, YOUR LAN ROX/, LUKE, EVERY1 ELSE FROM MERSEBURG-INF2K \
AND ALL OTHER PEOPLE WATCHING THOSE WEIRD STUFF...";
int				g_iScrollerPos;


/////////////////////////////////////////////////////////////////////////////////////
//Grafikobjekte (ebenfalls global angelegt)
XSurface		g_xsurfBackground;
XSprite			g_xsBars;
XSprite			g_xsBalls;
XFont			g_xfFont;


Ball g_Balls(g_iNrOfBalls);



/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::putBar
/////////////////////////////////////////////////////////////////////////////////////
void putBar(int iNr, int iY)
{
	int iCountX;

	g_xsBars.m_lFrame=iNr;
	g_xsBars.m_lY=iY;

	for(iCountX=0;iCountX<1024;iCountX+=64)
	{
			g_xsBars.m_lX=iCountX;
			g_xsBars.blit(&g_xdDisplay);
	}
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::initGFX
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		ldt die Grafiken in den Speicher und schaltet sie wenn ntig sichtbar
//			von dem Moment an, wo die Grafiken geladen sind, belegen sie
//			VID/SYS-MEM, es empfiehlt sich daher, Grafiken zu entladen, wenn sie
//			lnger nicht gebraucht werden. (z.B. die MenGFX im Ingame)
/////////////////////////////////////////////////////////////////////////////////////
void initGFX(void)
{
	XLogging::log(" --- Grafiken werden geladen ---\n");

	g_xsurfBackground.initialize(&g_xdDisplay,"gfx\\back.bmp");

	g_xsBars.initialize(&g_xdDisplay,"gfx\\bars.spr");
	g_xsBars.m_bVisible=true;

	g_xsBalls.initialize(&g_xdDisplay,"gfx\\balls.spr");
	g_xsBalls.m_bVisible=true;

	g_xfFont.initialize(&g_xdDisplay,"gfx\\font.fnt");
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::prepareEnd
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		schmeisst die Grafiken wieder aus dem Speicher, und bereitet so das
//			Ende der Applikation vor
/////////////////////////////////////////////////////////////////////////////////////
void prepareEnd(void)
{
	g_xsurfBackground.uninitialize();
	g_xsBars.uninitialize();
	g_xsBalls.uninitialize();

	g_xfFont.uninitialize();

	XLogging::log(" --- Grafiken entladen ---\n");
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::recover
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		holt Surfaces zurck, die wird z.b. nach einem Taskwechsel ntig
/////////////////////////////////////////////////////////////////////////////////////
void recover(void)
{
	g_xdDisplay.restore();

	g_xsurfBackground.restore();
	g_xsBars.restore();
	g_xsBalls.restore();

	g_xfFont.restore();

}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::script
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		script fr die Animation der Kugeln
/////////////////////////////////////////////////////////////////////////////////////
void script(int iTimePassed)
{
	
	static int iScriptCounter=0;
	static int iAngle=0;
	static int iTimeScale=0;
	static int iDistance=0;
	static int iInfo=0;

	int		iCountX;
	int		iCountY;
	double	dTmp;
	int		iActualBall;


	// in der Scriptvar gucken, was grad geht
	switch (iScriptCounter)
	{
	case 0: //Rechteckfeld aufbauen
		iDistance=-1000000;
		for (iCountX=0;iCountX<31;iCountX++)
		for (iCountY=0;iCountY<31;iCountY++)
		{
			g_Balls.setPos(	iCountX+iCountY*31,
							100.0f*(iCountX-15),
							100.0f*(iCountY-15),
							1000.0f);
		}
		g_Balls.setFloor(50000);
		iScriptCounter++;
		break;
	case 1: //Anzeige nach 3 secs
		if (g_xtTimer.getTime()>3000)
		{
			iDistance=90000;
			iScriptCounter++;
		}
		break;
	case 2: //wegzoomem nach 5.5
		if (g_xtTimer.getTime()>5500)
		{
			iDistance=120000;
			iAngle=2500;
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 3: //drehen fr 2 s
		iAngle+=iTimePassed;
		if (g_xtTimer.getTime()>2000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 4: //drehen fr 0.3 s schnell andersrum
		iAngle-=iTimePassed*16;
		if (g_xtTimer.getTime()>300)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 5: //drehen fr 2 s
		iAngle+=iTimePassed;
		if (g_xtTimer.getTime()>2000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 6: //drehen fr 0.3 s schnell andersrum
		iAngle-=iTimePassed*16;
		if (g_xtTimer.getTime()>300)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 7: //drehen fr 1 s
		iAngle-=iTimePassed;
		if (g_xtTimer.getTime()>1000)
		{
			iInfo=1;
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 8: //drehen fr 10 s schnell mit zuflliger Richtung
		iAngle=iAngle+iInfo*iTimePassed*8;
		if (rand()%(iTimePassed*12)==iTimePassed) iInfo=-iInfo;
		if (g_xtTimer.getTime()>10000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 9: //4s starrer Winkel und Sinus Wellen Profil
		iDistance=125000;
		iAngle=2500;
		g_Balls.setFloor(50000);
		for (iCountX=0;iCountX<31;iCountX++)
		for (iCountY=0;iCountY<31;iCountY++)
		{
			dTmp=sqrt((iCountX-15)*(iCountX-15)+(iCountY-15)*(iCountY-15));
			g_Balls.setPos(	iCountX+iCountY*31,
							100.0f*(iCountX-15),
							100.0f*(iCountY-15),
							120.0f*(float)cos(dTmp*0.5)+1000.0f);
		}
		if (g_xtTimer.getTime()>4000)
		{
			iAngle=0;
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 10: //10s DropFX
		iAngle+=iTimePassed*2;
		for (iCountX=0;iCountX<31;iCountX++)
		for (iCountY=0;iCountY<31;iCountY++)
		{
			dTmp=sqrt((iCountX-15)*(iCountX-15)+(iCountY-15)*(iCountY-15));
			g_Balls.setPos(	iCountX+iCountY*31,
							100.0f*(iCountX-15),
							100.0f*(iCountY-15),
							(float)((25.0-dTmp)*13.0*cos(dTmp*0.5+(double)g_xtTimer.getTime()*0.008))+1000.0f);
		}
		if (g_xtTimer.getTime()>10000)
		{
			iScriptCounter++;
		}
		break;
	case 11: //4s DropFX ausklingen lassen, dann fallen lassen
		iAngle+=iTimePassed*2;
		for (iCountX=0;iCountX<31;iCountX++)
		for (iCountY=0;iCountY<31;iCountY++)
		{
			dTmp=sqrt((iCountX-15)*(iCountX-15)+(iCountY-15)*(iCountY-15));
			g_Balls.setPos(	iCountX+iCountY*31,
							100.0f*(iCountX-15),
							100.0f*(iCountY-15),
							(float)((25.0-dTmp)*0.00325*cos(dTmp*0.5+(double)g_xtTimer.getTime()*0.008))
							*(14000-g_xtTimer.getTime())+1000.0f);
		}
		if (g_xtTimer.getTime()>13950)
		{
			iTimeScale=100;
			g_Balls.setFloor(1200);
			g_Balls.setGravity(0.004f);
			for (iCountX=0;iCountX<g_iNrOfBalls;iCountX++)
			{
				g_Balls.setDelta(	iCountX,
									0.0f,
									0.0f,
									-1.0f);
			}
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 12: //6.8s fallen lassen und dann nuke 'em !!
		iAngle+=iTimePassed*2;
		if (g_xtTimer.getTime()>6800)
		{
			//prepare nukage
			for (iCountX=0;iCountX<g_iNrOfBalls;iCountX++)
			{
				dTmp=0.001*(double)(rand()%6283);
				g_Balls.setDelta(	iCountX,
							(float)sin(dTmp),
							(float)cos(dTmp),
							-2.5f+(float)sin(dTmp*999.9));
			}
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 13: //6s wegrollen 
		if (g_xtTimer.getTime()>6000)
		{
			//erste Kugel schmeissen
			iActualBall=g_Balls.getActualBall();
			g_Balls.setPos(	iActualBall,
								-2000.0f,
								-2000.0f,
								1000.0f);
			g_Balls.setDelta(	iActualBall,
								1.0f,
								1.0f,
								-3.0f);

			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 14: //12s anfangen Kugel zu schmeissen
		dTmp=0.00001*(double)(g_xtTimer.getTime()*g_xtTimer.getTime());
		if (rand()%14400<(int)dTmp)
		{
			iActualBall=g_Balls.getActualBall();
			g_Balls.setPos(		iActualBall,
							-2000.0f,
							-2000.0f,
							1000.0f);
			g_Balls.setDelta(	iActualBall,
								0.7f+0.006f*(float)(rand()%100),
								0.7f+0.006f*(float)(rand()%100),
								-3.0f);
		}
		if (g_xtTimer.getTime()>12000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 15: //10s lang Kugeln schmeissen
		for (iCountX=0;iCountX<iTimePassed;iCountX+=10)
		{
			iActualBall=g_Balls.getActualBall();
			g_Balls.setPos(		iActualBall,
							-2000.0f,
							-2000.0f,
							1000.0f);
			g_Balls.setDelta(	iActualBall,
								0.6f+0.008f*(float)(rand()%100),
								0.6f+0.008f*(float)(rand()%100),
								-3.0f+0.003f*(float)(rand()%100));
		}
		if (g_xtTimer.getTime()>10000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
			iAngle=iAngle%20000;
		}
		break;
	case 16: //Kugeln schmeissen bis fertig gedreht
		iAngle+=iTimePassed;
		for (iCountX=0;iCountX<iTimePassed;iCountX+=10)
		{
			iActualBall=g_Balls.getActualBall();
			g_Balls.setPos(		iActualBall,
							-2000.0f,
							-2000.0f,
							1000.0f);
			g_Balls.setDelta(	iActualBall,
								0.6f+0.008f*(float)(rand()%100),
								0.6f+0.008f*(float)(rand()%100),
								-3.0f+0.003f*(float)(rand()%100));
		}
		if (iAngle>7500)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 17: //6s wegrollen lassen
		if (g_xtTimer.getTime()>6000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
			g_Balls.setFloor(2000);
		}
		break;
	case 18: //15s rain
		iAngle+=iTimePassed;
		for (iCountX=0;iCountX<iTimePassed;iCountX+=10)
		{
			iActualBall=g_Balls.getActualBall();
			g_Balls.setPos(		iActualBall,
								0.0f,
								0.0f,
								-500.0f);
			dTmp=0.001*(double)(rand()%6283);
			g_Balls.setDelta(	iActualBall,
								0.2f*(float)((1+sin(137*dTmp))*sin(dTmp)),
								0.2f*(float)((1+sin(137*dTmp))*cos(dTmp)),
								2*(float)sin(dTmp*999.9));
		}
		if (g_xtTimer.getTime()>15000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 19: //2s auf BulletTime gehen
		iTimeScale=100-g_xtTimer.getTime()/25;
		if (g_xtTimer.getTime()>2000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
			iTimeScale=20;
		}
		break;
	case 20: //5s "BulletTime"
		if (g_xtTimer.getTime()>5000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 21: //6s "BulletTime" drehend
		iAngle+=iTimePassed*6;
		if (g_xtTimer.getTime()>6000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 22: //5s "BulletTime"
		if (g_xtTimer.getTime()>5000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
		}
		break;
	case 23: //0,8s auf NormalZeit gehen
		iTimeScale=20+g_xtTimer.getTime()/10;
		if (g_xtTimer.getTime()>800)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
			iTimeScale=100;
		}
		break;
	case 24: //10s rollen, dann runterfallen lassen
		if (g_xtTimer.getTime()>10000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
			g_Balls.setFloor(100000);
		}
		break;
	case 25: //3s fallen lassen, dann 3D-Bild aufbauen
		if (g_xtTimer.getTime()>3000)
		{
			iTimeScale=0;
			iAngle=0;
			iDistance=120000;
			g_xtTimer.setTime(0);
			iScriptCounter++;
			for (iCountX=0;iCountX<g_iNrOfBalls;iCountX++)
			{
				dTmp=(double)iCountX/152.9479;
				g_Balls.setPos(	iCountX,
									800.0f*(float)sin(dTmp*31.3),
									800.0f*(float)cos(dTmp*31.3),
									78.0f*(float)(int)(iCountX/31)-100.0f);
				g_Balls.setDelta(	iCountX,
									0.0f,
									0.0f,
									0.0f);
				g_Balls.setGravity(0.004f);
			}
		}
		break;
	case 26: //10s rotieren, dann runterfallen lassen
		iAngle+=5*iTimePassed;
		if (g_xtTimer.getTime()>10000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
			iTimeScale=100;
			for (iCountX=0;iCountX<g_iNrOfBalls;iCountX++)
			{
				dTmp=(double)iCountX/152.9479;
				g_Balls.setDelta(	iCountX,
									0.4f*(float)sin(dTmp*31.3),
									0.4f*(float)cos(dTmp*31.3),
									-2.0f+0.5f*(float)cos(dTmp*555.5));
			}
		}
		break;
	case 27: //3s fallen lassen, dann 3D-Bild aufbauen
		if (g_xtTimer.getTime()>3000)
		{
			iTimeScale=0;
			iAngle=0;
			iDistance=120000;
			g_xtTimer.setTime(0);
			iScriptCounter++;
			for (iCountX=0;iCountX<g_iNrOfBalls;iCountX++)
			{
				dTmp=(double)iCountX/152.9479;
				g_Balls.setPos(	iCountX,
									800.0f*(float)sin(dTmp*21.0),
									800.0f*(float)cos(dTmp*15.0),
									1200.0f*(float)cos(dTmp*0.5)+1100.0f);
				g_Balls.setDelta(	iCountX,
									0.0f,
									0.0f,
									0.0f);
				g_Balls.setGravity(0.004f);
			}
		}
		break;
	case 28: //10s rotieren, dann runterfallen lassen
		iAngle+=5*iTimePassed;
		if (g_xtTimer.getTime()>10000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
			iTimeScale=100;
			for (iCountX=0;iCountX<g_iNrOfBalls;iCountX++)
			{
				dTmp=(double)iCountX/152.9479;
				g_Balls.setDelta(	iCountX,
									0.4f*(float)sin(dTmp*21.0),
									0.4f*(float)cos(dTmp*15.0),
									-2.0f+0.5f*(float)cos(dTmp*555.5));
			}
		}
		break;
	case 29: //3s fallen lassen, dann 3D-Bild aufbauen
		if (g_xtTimer.getTime()>3000)
		{
			iTimeScale=0;
			iAngle=0;
			iDistance=120000;
			g_xtTimer.setTime(0);
			iScriptCounter++;
			for (iCountX=0;iCountX<31;iCountX++)
			for (iCountY=4;iCountY<35;iCountY++)
			{
				g_Balls.setPos(	iCountX+31*(iCountY-4),
								(float)(sin(iCountX*4.9-iCountY*0.00)*iCountY*iCountY),
								(float)(cos(iCountX*4.9+iCountY*0.00)*iCountY*iCountY),
								1200.0f+(float)(sin(iCountX*0.2+iCountY*0.13)*iCountY*iCountY));
				g_Balls.setDelta(	iCountX+31*(iCountY-4),
									0.0f,
									0.0f,
									0.0f);
				g_Balls.setGravity(0.004f);
			}
		}
		break;
	case 30: //10s rotieren, dann runterfallen lassen
		iAngle+=5*iTimePassed;
		if (g_xtTimer.getTime()>10000)
		{
			g_xtTimer.setTime(0);
			iScriptCounter++;
			iTimeScale=100;
			for (iCountX=0;iCountX<31;iCountX++)
			for (iCountY=4;iCountY<35;iCountY++)
			{
				g_Balls.setDelta(	iCountX+31*(iCountY-4),
									0.0005f*(float)(sin(iCountX*4.9-iCountY*0.00)*iCountY*iCountY),
									0.0005f*(float)(cos(iCountX*4.9+iCountY*0.00)*iCountY*iCountY),
									-2.0f+0.5f*(float)cos(iCountY*555.5));
			}
		}
		break;
	case 31: //3s fallen lassen, dann von corn beginnen
		if (g_xtTimer.getTime()>3000)
		{
			iTimeScale=0;
			iAngle=0;
			iDistance=120000;
			g_xtTimer.setTime(0);
			iScriptCounter=0;
		}
		break;
	}

	g_Balls.setDistance(iDistance);
	g_Balls.moveandtransform(iAngle,0.01f*(float)iTimePassed*iTimeScale);
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::mainloop
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		die eigentliche Hauptschleife des Programms
/////////////////////////////////////////////////////////////////////////////////////
void mainloop(void)
{
	static double dBarTimer=0;
	int iTimePassed;

	iTimePassed=g_xtTimer.update();

	// Hintergrund blitten
	g_xsurfBackground.blit(&g_xdDisplay);

	//Ballscript laufen lassen
	script(iTimePassed);

	//Blle malen
	g_Balls.draw(&g_xdDisplay,&g_xsBalls,32);


	// Zeit fr Scrolleranim holen
	dBarTimer+=iTimePassed/500.0;

	//wenn Bars  vorn sind zuerst den Text malen
	if (cos(dBarTimer)<0)
		g_xfFont.textOut(&g_xdDisplay,g_iScrollerPos/16,368,g_pcScrollText);

	//Richtung der Bars checken und malen
	if (sin(dBarTimer)>0)
	{
		putBar(0,368+(int)(185.0*sin(-0.6+dBarTimer)));
		putBar(1,368+(int)(185.0*sin(-0.3+dBarTimer)));
		putBar(2,368+(int)(225.0*sin(0.0+dBarTimer)));
		putBar(3,368+(int)(185.0*sin(0.3+dBarTimer)));
		putBar(0,368+(int)(185.0*sin(0.6+dBarTimer)));
	}
	else
	{
		putBar(0,368+(int)(185.0*sin(0.6+dBarTimer)));
		putBar(3,368+(int)(185.0*sin(0.3+dBarTimer)));
		putBar(2,368+(int)(185.0*sin(0.0+dBarTimer)));
		putBar(1,368+(int)(185.0*sin(-0.3+dBarTimer)));
		putBar(0,368+(int)(185.0*sin(-0.6+dBarTimer)));
	}

	// wenn Bars hinten sind zuletzt den Text malen
	if (cos(dBarTimer)>=0)
		g_xfFont.textOut(&g_xdDisplay,g_iScrollerPos/16,368,g_pcScrollText);
	
	// Scroller weiterschieben
	g_iScrollerPos-=iTimePassed*4;
	if (g_iScrollerPos<-200000)g_iScrollerPos=20000;

}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::run
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		diese Funktion wird von der main immer aufgerufen, wenn die Anwendung den
//			Fokus hat, und nichts anderes zu tun ist
/////////////////////////////////////////////////////////////////////////////////////
void run(void)
{
	// Mainloop rufen/Bild aufbauen
	mainloop();

	// evtl. FrameCounter anzeigen
	if (g_bFCounterOn) g_xfcFrameCounter.blit(&g_xdDisplay); 

	// Bild auf den Screen flippen
	HRESULT hResult	= g_xdDisplay.flip();

	// Bei verlorenen Surfaces, die Surfaces wiederholen
	if (hResult==DDERR_SURFACELOST)
	{
		recover();
	}

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::wndProc
/////////////////////////////////////////////////////////////////////////////////////
// Verarbeitung der Windows Messages
/////////////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK wndProc(HWND hWnd,
						 UINT message,
						 WPARAM wParam,
						 LPARAM lParam)
{
	switch (message)
	{
		case WM_KEYDOWN:
			switch (wParam)
			{
				case 'F':
					g_bFCounterOn=!g_bFCounterOn;
					break;
					
				case VK_ESCAPE:
					PostMessage(hWnd,WM_CLOSE,0,0);
					g_bESCKey=true;
					break;
			}
			break;
		case WM_CLOSE:
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			// Bei Fokus-Verlust die Ausfuehrung der Anwendung anhalten
			if ((wParam==SIZE_MAXHIDE) || (wParam==SIZE_MINIMIZED))
			{
				g_bRunning	= false;
			}
			else // sonst Ausfuehrung fortsetzen
			{
				g_bRunning	= true;
			}
			break;
	} // switch

	if (g_bWantToQuit) PostMessage(hWnd,WM_CLOSE,0,0);

	return ::DefWindowProc(hWnd,message,wParam,lParam);
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::initialize
/////////////////////////////////////////////////////////////////////////////////////
//INFO:	initialisiert alles, wird zu Beginn des Programms von der main aufgerufen
/////////////////////////////////////////////////////////////////////////////////////
HRESULT initialize(HINSTANCE	hInstance)
{
	HRESULT	hRes;

   	// Anwendungsfenster erzeugen
	g_xaApplication.initialize(hInstance,"XGamesSDK Sample","XGamesSDK Sample",IDI_ICON1);
	
	// Timer initialisieren
	g_xtTimer.setUp();
	
	// FrameCounter initialisieren
	g_xfcFrameCounter.setFont(&g_xfFont);
	g_xfcFrameCounter.setText("FPS: $f4");
	g_xfcFrameCounter.m_lX	= 0;
	g_xfcFrameCounter.m_lY	= 0;

	//Scroller und Script auf Angang setzen
	g_iScrollerPos=20000;

	// Programm auf "luft" schalten
	g_bRunning=true;
	
	// Grafikmodi initialiseren
	// wenn einer nicht geht, wird der nchst niedere probiert, bis einer gefunden wird

	hRes=g_xdDisplay.initialize(1024,768,32,g_xaApplication.getHWnd(),DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN,false);
	if (hRes==DD_OK)
	{
		XLogging::log("1024x768x32 initialized...\n");
		return hRes;
	}
	XLogging::log("1024x768x32 failed...\n");

	g_xdDisplay.uninitialize();
	
	hRes=g_xdDisplay.initialize(1024,768,16,g_xaApplication.getHWnd(),DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN,false);
	if (hRes==DD_OK)
	{
		XLogging::log("1024x768x16 initialized...\n");
		return hRes;
	}
	XLogging::log("1024x768x16 failed...\n");
	g_xdDisplay.uninitialize();
	
	hRes=g_xdDisplay.initialize(1024,768,15,g_xaApplication.getHWnd(),DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN,false);
	if (hRes==DD_OK)
	{
		XLogging::log("1024x768x15 initialized...\n");
		return hRes;
	}
	XLogging::log("1024x768x15 failed...\n");
	XLogging::log("Error!!! This videocard is not supported\n");
	
	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
//INFO: Gegenstck zu initialize, wird am Ende der main aufgerufen
/////////////////////////////////////////////////////////////////////////////////////
HRESULT unintialize(void)
{
	HRESULT	hRes	= DD_OK;

	// DirectX deinitialisieren
	g_xdDisplay.uninitialize();
	
	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::WinMain
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		Einsprungpunkt des Programms
/////////////////////////////////////////////////////////////////////////////////////
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG			msg;
	HRESULT		hRes;

	//Programm will erstmal nicht beenden (!)
	g_bWantToQuit=false;
	
	// Logging anwerfen
	XLogging::setContext(true,true,false,"error.log");
	XLogging::log("\n--- Programmstart ---\n\n");

	// Programm-Startup
	hRes=::initialize(hInstance);

	//keine Grafikmode funzt? Programm will beenden!
	if (hRes!=DD_OK)
	{
		g_bWantToQuit=true;	
	}

	// Grafiken laden
	initGFX();

	while (1)
	{
		if (::PeekMessage(&msg,NULL,0,0,PM_NOREMOVE))
		{
			if (!::GetMessage(&msg,NULL,0,0))
			{ // WM_QUIT bekommen
				XLogging::log("\nProgrammende eingeleitet...\n");
				
				// Grafiken entladen
				prepareEnd();

				// while-Schleife beenden
				break;
			}
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}
		else
		{
			if (g_bRunning==true)
			{ // wenn die Anwendung den Fokus hat Programm ausfuehren
				run();

			}
		}
	}

	// Programm-Shutdown
	::unintialize();

	XLogging::log("\n--- Programmende ---\n\n");

	return msg.wParam;
}


/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	main.cpp
/////////////////////////////////////////////////////////////////////////////////////
